import java.io.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

/**
 * La classe <code>Drawings</code> permet de r&eacute;aliser des dessins.
 * Elle permet de masquer certaines constructions de Java que nous avons
 * consid&eacute;r&eacute;es comme trop avanc&eacute;es pour une initiation &agrave; la programmation.
 * D'autre part, une tentative d'uniformisation de nommage a &eacute;t&eacute; faite afin
 * de normaliser les incantations Java.
 *
 * <P>Un exemple de programme graphique typique est le suivant:
 * <PRE>
 * Drawings.startDrawings(200,200);
 * for(int l=1; l<100; l+=10)
 *   for(int alpha=0; alpha<360; alpha++)
 *     Drawings.drawPoint((int)(100+l*Math.cos(alpha*Math.PI/180)),
 *                        (int)(100+l*Math.sin(alpha*Math.PI/180)));
 * </PRE>
 * Pour obtenir:<BR>
 * <IMG SRC="Dessin.jpg">
 */
public final class Drawings {
    /**
     * Pas d'instanciation de la classe <code>Drawings</code>
     */
    private Drawings() {
    }
    /**
     * R&eacute;f&eacute;rence sur le flot de d&eacute;coupage en lex&egrave;mes.
     */
    private static JFrame theFrame = null;
    private static Drawable theDrawable = null;
    /**
     * Pr&eacute;pare une zone d'affichage de taille 200x200.
     */
    public static void startDrawings() {
	startDrawings(200,200);
    }
    /**
     * Pr&eacute;pare une zone d'affichage de taille arbitraire.
     * @param width La largeur de la zone d'affichage
     * @param height La hauteur de la zone d'affichage
     */
    public static void startDrawings(int width,int height) {
	if (theFrame!=null) return;
	theFrame = new JFrame("Dessin");
	theFrame.setResizable(false);
	theDrawable = new Drawable(width,height);
	theFrame.getContentPane().add(theDrawable);
	theFrame.pack();
	theFrame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
	theFrame.setVisible(true);
	try {
	    Thread.sleep(1000); // Should be synchronized!!!
	} catch(Exception e) {
	}
    }
    /**
     * Ferme la fen&ecirc;tre graphique.
    */
    private static void stopDrawings() {
	if (theFrame==null) return;
	theFrame.dispose();
	theFrame=null;
    }
    /**
     * Dessine un point dans la zone d'affichage.
     * @param x L'abscisse du point.
     * @param y L'ordonn&eacute;e du point.
     */
    public static void drawPoint(int x,int y) {
	if (theFrame==null)
	    throw new RuntimeException("il manque un appel &agrave; startDrawings()");
	theDrawable.drawLine(x,y,x,y);
    }
    /**
     * Modifie la couleur de l'encre des points &agrave; dessiner.
     * Les couleurs disponibles sont au nombre de 256 <em>niveaux</em> de gris,
     * de <code>0</code> pour coder le noir &agrave; <code>255</code> pour
     * coder le blanc. Un gris m&eacute;dian s'obtient donc avec <code>127</code>.
     * @param c La couleur.
     */
    private static void setGray(int c) {
	if (theFrame==null)
	    throw new RuntimeException("il manque un appel &agrave; startDrawings()");
	theDrawable.setColor(c,c,c);
    }
    /**
     * Dessine un segment de droite dans la zone d'affichage.
     * @param x0 L'abscisse du premier point.
     * @param y0 L'ordonn&eacute;e du premier point.
     * @param x1 L'abscisse du second point.
     * @param y1 L'ordonn&eacute;e du second point.
     */
    private static void drawLine(int x0,int y0,int x1, int y1) {
	if (theFrame==null)
	    throw new RuntimeException("il manque un appel &agrave; startDrawings()");
	theDrawable.drawLine(x0,y0,x1,y1);
    }


    /**
     * Dessine un rectangle vide dans la zone d'affichage.
     * <p>
     * Les droites d&eacute;limitant le rectangle &agrave; gauche et &agrave; droite sont
     * situ&eacute;es aux abscisses <code>x</code> et <code>x+l</code>.
     * Celles d&eacute;limitant le rectangle en haut et en bas sont situ&eacute;es
     * aux ordonn&eacute;es <code>y</code> et <code>y+h</code>.
     * @param x L'abscisse du coin nord-ouest du rectangle.
     * @param y L'ordonn&eacute;e du coin nord-ouest du rectangle.
     * @param l La largeur du rectangle.
     * @param h La hauteur du rectangle.
     */
    private static void drawRect(int x,int y,int l, int h) {
	if (theFrame==null)
	    throw new RuntimeException("il manque un appel &agrave; startDrawings()");
	theDrawable.drawRect(x,y,l,h);
    }

    /**
     * Dessine un rectangle plein dans la zone d'affichage.
     * <p>
     * Les droites d&eacute;limitant le rectangle &agrave; gauche et &agrave; droite sont
     * situ&eacute;es aux abscisses <code>x</code> et <code>x+l</code>.
     * Celles d&eacute;limitant le rectangle en haut et en bas sont situ&eacute;es
     * aux ordonn&eacute;es <code>y</code> et <code>y+h</code>.
     * @param x L'abscisse du coin nord-ouest du rectangle.
     * @param y L'ordonn&eacute;e du coin nord-ouest du rectangle.
     * @param l La largeur du rectangle.
     * @param h La hauteur du rectangle.
     */
    private static void fillRect(int x,int y,int l, int h) {
	if (theFrame==null)
	    throw new RuntimeException("il manque un appel &agrave; startDrawings()");
	theDrawable.fillRect(x,y,l,h);
    }

    /**
     * Dessine un cercle vide dans la zone d'affichage.
     * @param x L'abscisse du centre du cercle.
     * @param y L'ordonn&eacute;e du centre du cercle.
     * @param r Le rayon du cercle.
     */
    private static void drawCircle(int x,int y,int r) {
	if (theFrame==null)
	    throw new RuntimeException("il manque un appel &agrave; startDrawings()");
	theDrawable.drawOval(x-r, y-r, 2*r, 2*r);
    }

    /**
     * Dessine un cercle plein dans la zone d'affichage.
     * @param x L'abscisse du centre du cercle.
     * @param y L'ordonn&eacute;e du centre du cercle.
     * @param r Le rayon du cercle.
     */
    private static void fillCircle(int x,int y,int r) {
	if (theFrame==null)
	    throw new RuntimeException("il manque un appel &agrave; startDrawings()");
	theDrawable.fillOval(x-r, y-r, 2*r, 2*r);
    }


    /**
     * Dessine un arc de cercle vide dans la zone d'affichage.
     * <p>
     * La valeur des angles est sp&eacute;cifi&eacute;e en degr&eacute;.
     * Le sens de rotation est celui des cercles trigonom&eacute;triques.
     * L'angle de d&eacute;part est <code>angleInit</code>
     * et l'angle de fin <code>angleInit + angleCouvert</code>
     * @param x L'abscisse du centre du cercle.
     * @param y L'ordonn&eacute;e du centre du cercle.
     * @param r Le rayon du cercle.
     * @param angleInit Angle de d&eacute;part.
     * @param angleCouvert Angle de rotation.
     */
    private static void drawArc(int x,int y,int r, int angleInit, int angleCouvert) {
	if (theFrame==null)
	    throw new RuntimeException("il manque un appel &agrave; startDrawings()");
	theDrawable.drawArc(x-r, y-r, 2*r, 2*r, angleInit, angleCouvert);
    }

    /**
     * Dessine un arc de cercle plein dans la zone d'affichage.
     * <p>
     * La valeur des angles est sp&eacute;cifi&eacute;e en degr&eacute;.
     * Le sens de rotation est celui des cercles trigonom&eacute;triques.
     * L'angle de d&eacute;part est <code>angleInit</code>
     * et l'angle de fin <code>angleInit + angleCouvert</code>
     * @param x L'abscisse du centre du cercle.
     * @param y L'ordonn&eacute;e du centre du cercle.
     * @param r Le rayon du cercle.
     * @param angleInit L'angle de d&eacute;part.
     * @param angleCouvert L'angle de rotation.
     */
    private static void fillArc(int x,int y,int r, int angleInit, int angleCouvert) {
	if (theFrame==null)
	    throw new RuntimeException("il manque un appel &agrave; startDrawings()");
	theDrawable.fillArc(x-r, y-r, 2*r, 2*r, angleInit, angleCouvert);
    }

    /**
     * Efface int&eacute;gralement le contenu de la fen&ecirc;tre graphique.
    */
    private static void clearArea() {
	if (theFrame==null)
	    throw new RuntimeException("il manque un appel &agrave; startDrawings()");
	theDrawable.clearArea();
    }
}

class Drawable extends Canvas {
    private int width, height;
    private Image backingStore;
    private Graphics backingGraphics;
    public Drawable(int width,int height) {
	setSize(width,height);
    }
    public void setSize(int width,int height) {
	super.setSize(width,height);
	this.width = width;
	this.height = height;
    }
    public Dimension getPreferredSize() {
	return new Dimension(width,height);
    }
    public void paint(Graphics g) {
	if (backingStore==null) {
	    backingStore = createImage(Drawable.this.width,
				       Drawable.this.height);
	    backingGraphics = backingStore.getGraphics();
	}
	g.drawImage(backingStore,0,0,this);
    }
    public void update(Graphics g) {
	paint(g);
    }
    public void drawLine(int x0,int y0,int x1, int y1) {
	if (backingGraphics==null) return;
	backingGraphics.drawLine(x0,y0,x1,y1);
	repaint();
    }
    
    public void drawRect(int x, int y, int l, int h) {
	if(backingGraphics==null) return;
	backingGraphics.drawRect(x,y,l,h);
	repaint();
    }

    public void fillRect(int x, int y, int l, int h) {
	if(backingGraphics==null) return;
	backingGraphics.fillRect(x,y,l,h);
	repaint();
    }

    public void drawOval(int x, int y, int l, int h) {
	if(backingGraphics==null) return;
	backingGraphics.drawOval(x,y,l,h);
	repaint();
    }

    public void fillOval(int x, int y, int l, int h) {
	if(backingGraphics==null) return;
	backingGraphics.fillOval(x,y,l,h);
	repaint();
    }

    public void drawArc(int x, int y, int l, int h, int angleInit, int angleCouvert) {
	if(backingGraphics==null) return;
	backingGraphics.drawArc(x,y,l,h, angleInit, angleCouvert);
	repaint();
    }

    public void fillArc(int x, int y, int l, int h, int angleInit, int angleCouvert) {
	if(backingGraphics==null) return;
	backingGraphics.fillArc(x,y,l,h, angleInit, angleCouvert);
	repaint();
    }

    public void setColor(int c1,int c2,int c3) {
	if (backingGraphics==null) return;
	backingGraphics.setColor(new Color(c1,c2,c3));
    }
    public void clearArea() {
	if (backingGraphics==null) return;
	backingGraphics.clearRect(0,0,width,height);
	repaint();
    }
    public void sleep(int secondes) {
        try {
            Thread.sleep(secondes*100);
        } catch(InterruptedException ie) {}
    }
}

